
    // Character.cpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Jueves 02 De Mayo Del 2013    


#include "Character.hpp"

Character::Character(string name,string model_name, Location *location, float initial_angle){
  this->model_name = model_name;
  this->location = location;
  this->ent_name = name;
  scene_node = NULL;

  step = 2;
  angle_step = 0.01;
  angle = 0;
  this->initial_angle = initial_angle;
  
}

Character::~Character(){
}


void Character::load(Ogre::SceneManager *smgr){
  string model_path = CHARACTER_PATH;
  model_path = model_path + "/" + this->model_name;  

  Message::show("C", "Loading character model:");
  Message::show("C", model_path);

  entity = smgr->createEntity(ent_name, model_path);
 
  scene_node = smgr->getRootSceneNode()->createChildSceneNode();
  scene_node->attachObject(entity);
  
  /** 
      @todo Remove the PC translation. It was used for debuging the character drawing.
  */
  scene_node->translate(0, -20, 0);
  
 
  set_angle(angle);
  
  // Set initial position.
  //location->set_initial_position(node);

}

float* Character::get_position(){
  return (float*) scene_node->getPosition().ptr();
}

void Character::set_position(float pos[3]){
  scene_node->setPosition(Ogre::Vector3(pos));
}

Location* Character::get_location(){
  return location;
}

void Character::set_location(Location *loc){
  if (location != NULL){
    delete(location);
  }
  
  location = loc;
}


float Character::get_angle(){
  return angle;
}

void Character::set_angle(float newangle){
  angle = newangle;
  
  scene_node->yaw(Ogre::Radian(angle));

}

float Character::get_step(){
  return step;
}

void Character::set_step(float newstep){
  step = newstep;
}

float Character::get_angle_step(){
  return angle_step;
}

void Character::set_angle_step(float angle){
  this->angle_step = angle;
}

string Character::get_name(){
  return ent_name;
}


void Character::forward(){
#ifdef __CHARACTER_DEBUG
  Message::show("C","Moving Forward");
#endif

  scene_node->translate(0,0,-step);
}

void Character::backwards(){
#ifdef __CHARACTER_DEBUG
  Message::show("C","Moving Backward");
#endif

  scene_node->translate(0,0,step);
}

void Character::left(){
#ifdef __CHARACTER_DEBUG
  Message::show("C","Moving Left");
#endif

  scene_node->translate(-step,0,0);
}

void Character::right(){
#ifdef __CHARACTER_DEBUG
  Message::show("C","Moving Right");
#endif

  scene_node->translate(step,0,0);
}

void Character::rot_left(){  
#ifdef __CHARACTER_DEBUG
  Message::show("C", "Rotating left");
#endif

  scene_node->yaw(Ogre::Radian(angle_step));
}

void Character::rot_right(){
#ifdef __CHARACTER_DEBUG
  Message::show("C", "Rotating right");  
#endif

  scene_node->yaw(Ogre::Radian(-angle_step));
}


float Character::get_initial_angle(){
  return initial_angle;
}

void Character::play_animation(string name){
  
}

void Character::stop_animation(){
  
}
