/*
  InitFile.hpp
  Copyright (C) 2013  Giménez, Christian N.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  jueves 24 de enero del 2013     */


#ifndef __INITFILE_HPP
#define __INITFILE_HPP 1

#define INITCONFIGFILE = "initial.conf"

#include <string>
#include <iostream>
#include <fstream>

using namespace std;

#include "InitialConfig.hpp"
#include "Terminal.hpp"

using namespace Terminal;

namespace Loaders{

  /**
     I'm responsible for loading the initial configuration.
     
     I create an instance of InitialConfig filled with all the information that there is in the initial configuration file.
  */
  class InitFile {

  protected:
    /**
       This is the instance of InitialConfig that I am creating.
    */
    InitialConfig* config;

    /**
       This is the filename I'll open.
     */
    string filename;

    /**
       The streamming that I use for loading the information.
     */
    ifstream file;

  public:
    /**
       Create an instance of InitFile with the default filename.

       The default filename is given by the #define declaration @c INITCONFIGFILE in InitFile.hpp.
     */
    InitFile();
    /**
       Create an instance of InitFile wich use the given filename.
       
       @param file An instance of string with the filename.
     */
    InitFile(string file);
    ~InitFile();

    /**
       I load the file and create the instance of InitialConfig.

       @retval Null if nothing could be loaded.
       @return An instance of InitialConfig if everything is fine.
     */
    InitialConfig* load();

  };

}
#endif /* __INITFILE_HPP */

