
/*
  Light.cpp
  Copyright (C) 2013  Giménez, Christian N. 
  
  This program is free software: you can redistribute it and/or modify 
  it under the terms of the GNU General Public License as published by 
  the Free Software Foundation, either version 3 of the License, or 
  (at your option) any later version. 
  
  This program is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURDIRE.  See the 
  GNU General Public License for more details. 
  
  You should have received a copy of the GNU General Public License 
  along with this program.  If not, see <http://www.gnu.org/licenses/>. 
  
  sábado 02 de febrero del 2013    */

#include "Light.hpp"

#include <string>

using namespace std;

#include "Terminal.hpp"

using namespace Terminal;

Light::Light(string name){
  this->name = name;

  dir[0] = 0.55;
  dir[1] = -0.3;
  dir[2] = 0.75;
  
  color[0] = 1.0;
  color[1] = 1.0;
  color[2] = 1.0; 

  type = DIRECTIONAL;

  lightnode = NULL;
}

Light::~Light(){
}

Light::LightType Light::get_type(){
  return type;
}

void Light::set_type(LightType type){
  this->type = type;

  refresh_lightnode_data();  
}

void Light::set_dir(float* newdir){
  dir[0] = newdir[0];
  dir[1] = newdir[1];
  dir[2] = newdir[2];

  refresh_lightnode_data();  
}

void Light::set_color(float* newcolor){
  color[0] = newcolor[0];
  color[1] = newcolor[1];
  color[2] = newcolor[2];

  refresh_lightnode_data();
  
}

float* Light::get_dir(){
  return dir;
}

float* Light::get_color(){
  return color;
}

Ogre::Light* Light::get_lightnode(){
  return lightnode;
}


void Light::load(Ogre::SceneManager *smgr){
  lightnode = smgr->createLight(name);
  
  refresh_lightnode_data();
}

void Light::refresh_lightnode_data(){
  if (lightnode == NULL){
    return;
  }

  Ogre::Light::LightTypes t;
  
  switch (type){
  case POINT:
    t = Ogre::Light::LT_POINT;
    break;
  case DIRECTIONAL:
    t = Ogre::Light::LT_DIRECTIONAL;
    break;
  case SPOTLIGHT:
    t = Ogre::Light::LT_SPOTLIGHT;
    break;
  };
  lightnode->setType(t);
  lightnode->setDirection(Ogre::Vector3(dir));
  lightnode->setDiffuseColour(Ogre::ColourValue::White);  
  lightnode->setSpecularColour(Ogre::ColourValue(color[0], color[1], color[2]));
  
}
