/*
  Light.hpp
  Copyright (C) 2013  Giménez, Christian N.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  sábado 02 de febrero del 2013     */


#ifndef __LIGHT_HPP
#define __LIGHT_HPP 1

#include <string>

using namespace std;

#include <OgreRoot.h>
#include <OgreSceneManager.h>

/**
   I represent a dynamic light in the scene.
 */
class Light {

public:
  /**
     These are the type of lights.
   */
  enum LightType {POINT, DIRECTIONAL, SPOTLIGHT};

protected:
  /**
     Direction of the light.
   */
  float dir[3];
  
  float color[3];

  /**
     My type of light.
   */
  LightType type;

  /**
     My name.
   */
  string name;

  /**
     After loading this is setted for Ogre manipulation.
   */
  Ogre::Light* lightnode;

public:

  /**
     I create an instance of me with a given name.

     @param name An instance of string.
   */
  Light(string name);
  ~Light();

  /**
     @name Getters and Setters.
   */
  //@{
  /**
     Set the type of light that I should be.
     
     @param type An enumeration, can be one of the following: POINT, DIRECTIONAL, SPOTLIGHT.
     
   */
  void set_type(LightType type);
  /**
     Return the type of light.
     @returns
     One of the following: POINT, DIRECTIONAL or SPOTLIGHT.
   */
  LightType get_type();

  void set_dir(float* newdir);
  void set_color(float* newcolor);

  float* get_dir();
  float* get_color();

  /**
     Return the lightnode associated.
     
     @return 
     An instance of Ogre::Light asociated when loaded into the SceneManager.
     
     If I am not initialized, then I return NULL.
   */
  Ogre::Light* get_lightnode();
  //@}

  /**
     Add the light into the SceneManager instance.

     @param smgr An instance of SceneManager.
   */
  void load(Ogre::SceneManager *smgr);

protected:
  
  /**
     I set all the information I have about the light into the Ogre::Light instance(if there is one created by the load() function).
   */
  void refresh_lightnode_data();

};


#endif /* __LIGHT_HPP */

