
    // Location.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#ifndef __LOCATION_HPP__
#define __LOCATION_HPP__

#include <vector>
#include <string>

using namespace std;

#include <OgreRoot.h>
#include <OgreSceneManager.h>
#include <OgreEntity.h>
#include <OgreSceneNode.h>

// using namespace Ogre;

#include "Light.hpp"

/**
   I am an abstraction of a Location. A location is a place on the world where you can walk and do things! 
  
   The model path is determined by the name. We use 3ds files.
 */
class Location{

protected:
  /** 
      My name :)
   */
  string name;
  
  /**
     Model filename of the location.
  */
  string modelfile;

  /**
     This is a collection of lights.
   */
  vector<Light*> col_lights;

  /**
     This is the character starting position.
     Is a vector3df.
   */
  //vector3df starting_pos;

  Ogre::Entity* entity;

  Ogre::SceneNode* scene_node;
  
public:
  /**
     I create an instance of a location. 
          
     @param name A string Name of the new instance.
     @param mode_filename A string giving the model filename.
   */
  Location(string name, string model_filename);
  //Location(string name, string model, vector3df starting_pos);
  ~Location();
  
  /**
     Load the location into the SceneManager! :)

     Entities and SceneNodes are created here.

     @param smgr Is a pointer to Ogre::SceneManager.
   */
  void load(Ogre::SceneManager *smgr);

  /**
     Set the path of the texture that use this scene.
   */
  // void setTexturename(string path);
  // string getTexturename();


  /**
     @name Lights

     Functions for managing my lights sources.

  */
  //@{
  /** 
      Add a preelaborated Light.

      @param l An instance of Light.
  */
  void add_light(Light* l);

  /**
     I create a light for you and add it!

     @param name I need the light name, is an instance of string. Be sure to not repeat the name with other lights and Ogre::SceneNodes(Locations, things, etc.)
     @param dir I need the direction, is an array of  three floats representing a Math vector.
     @param color The color, is an array of three float representing the RGB color table.
   */
  void add_light(string name, float dir[3], float color[3]);

  //@}

  /**
     Set the character starting position.
     @param pos Is a vector3df that set the starting position.
   */
  //void setStart(vector3df pos);
 
  /**
     Return the character starting position.
     @return A vector3df that is the starting position of the character.
   */
  //  vector3df getStart();


  /**
     Return the node.

     @return An instance of IMeshSceneNode that is the node of this location.
   */
  //IMeshSceneNode* get_node();

  /**
     Set the position to the node at the initial position of this Location.

     @param An instance of ISceneNode that will be changed its position.
   */
  //void set_initial_position(ISceneNode* node);

protected:

  /**
     @name Lights
  */
  //@{
  /**
     I load all the lights into the given Ogre::SceneManager instance.
   */
  void load_lights(Ogre::SceneManager *smgr);
  //@}


};

#endif
