
    // LocationFile.cpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#include <string>

using namespace std;

#include "LocationFile.hpp"

using namespace Loaders;

#include "Terminal.hpp"

using namespace Terminal;

#include "common-functions.hpp"

/*********************
Constructors and destructor
*********************/


// TODO: Remove this constructor... only get more difficult with it...
LocationFile::LocationFile(std::string filename){
  this->filename = filename;
  opened = false;

  builder = NULL; // This is a problem...
}

LocationFile::LocationFile(LocationBuilder *builder, std::string worldname, std::string loc_name){
  filename = LF_PATH;
  filename = filename + worldname + "/";
  filename = filename + loc_name + "/";
  filename = filename + loc_name;
  
  this->builder = builder;

  opened = false;
}


LocationFile::~LocationFile(){
}



/*********************/



std::string LocationFile::next(){
  if (!opened){
    // File was not opened!!!
    Error::show("LF", "File is not open!");

    next_type=LF_ENDFILE;
    return std::string(LF_ENDFILE);
  }

  if (file.eof()){    
    // Problems reached: no more data...    
    close();

    next_type = LF_ENDFILE;
    return std::string(LF_ENDFILE);
  }

  // No problems... Continue :)
  
  Message::show("LF", "Loading type:");

  char buffer[200];
  file.getline(buffer, 200);
  
  next_type = CommonFuncs::trim(string(buffer));
  

  Message::show("LF", next_type.c_str()); 
  
  if (next_type.substr(0,5) == "Begin"){
    // take out the "Begin " firsts characters.
    next_type = next_type.substr(6);
    Message::show("LF", next_type.c_str());
  } 

 
  load_element();

  // Get the "End" string.
  /*std::string ending;
    file.getline(buffer,200);
    ending = string(buffer).trim().c_str();
    if (ending != "End"){    
    Warning::show("LF", "Premature ending! No \"End\" after \"Begin *\" in location file.");
    return std::string(LF_ENDFILE);
    }*/

  Message::show("LF", "Finish loading type.");
  return next_type;
}


/*********************
Loading and getting elements
*********************/

void LocationFile::load_element(){
  // TODO: Remove "if(builder!=NULL)..." when removing constructor LocationFile(std::string filename).
  
  // Is the location first data
  if (next_type == LF_LOCATION){    
    load_name();
    load_model_name();
    //load_texture_name();
    if (builder != NULL){
      builder->build_location(name,model);
      // TODO: Enable texture loading...
      // builder->build_texture(texture);
    }
  }

  // Is a starting position
  // if (next_type == LF_START){
  //   load_starting_pos();
  //   if (builder != NULL){
  //     builder->build_starting(startpos);
  //   }
  // }

  // Is a light
  if (next_type == LF_LIGHT){
    if (!load_light_name()){
      return;
    }
    if (!load_light_dir()){
      return;
    }
    if (!load_light_color()){
      return;
    }
    if (builder != NULL){
      builder->build_light(lname, ldir, lcolor);
    }
  }
}

bool LocationFile::load_light_dir(){
  Message::show("LF", "load_light_dir()");

  if (!opened || file.eof()){
    return false;
  }

  float x, y, z;
  file >> x >> y >> z;
  ldir[0] = x;
  ldir[1] = y;
  ldir[2] = z;

  Message::show("LF", x);
  Message::show("LF", y);
  Message::show("LF", z);

  return true;
}

float* LocationFile::get_light_dir(){
  return ldir;
}

bool LocationFile::load_light_color(){
  Message::show("LF", "load_light_color()");

  if (!opened || file.eof()){
    return false;
  }

  float r,g,b,a;
  file >> r >> g >> b >> a;
  
  lcolor[0] = r;
  lcolor[1] = g;
  lcolor[2] = b;
  lcolor[3] = a;

  Message::show("LF",r);
  Message::show("LF",g);
  Message::show("LF",b);
  Message::show("LF",a);
  return true;
}

float* LocationFile::get_light_color(){
  return lcolor;
}

bool LocationFile::load_light_name(){
  Message::show("LF", "load_light_name()");

  if (!opened || file.eof()){
    return false;

  }

  char buffer[200];
  file.getline(buffer,200);

  lname = CommonFuncs::trim(string(buffer));

  Message::show("LF",lname.c_str());
  return true;
}

string LocationFile::get_light_name(){
  return lname;
}
 
// void LocationFile::load_starting_pos(){
//   float x,y,z;
//   file >> x >> y >> z;
  
//   startpos = vector3df(x,y,z);

//   Message::show("LF",string(x));
//   Message::show("LF",string(y));
//   Message::show("LF",string(z));
// }

// vector3df LocationFile::get_starting_pos(){
//   return startpos;
// }
  
void LocationFile::load_name(){
  Message::show("LF", "load_name()");
  char buffer[200];

  file.getline(buffer,200);

  name = string(buffer);  
  
  Message::show("LF",name);
}

string LocationFile::get_name(){
  return name;
}


void LocationFile::load_model_name(){
  Message::show("LF", "load_model_name()");
  char buffer[200];
  file.getline(buffer,200);
  
  this->model = string(buffer);

  Message::show("LF",this->model);
}

string LocationFile::get_model_name(){
  return model;
}

// void LocationFile::load_texture_name(){
//   char buffer[200];
//   file.getline(buffer,200);
  
//   this->texture = string(buffer);

//   Message::show("LF",this->texture);
// }

// string LocationFile::get_texture_name(){
//   return texture;
// }



/*********************
For using with Builder
*********************/
void LocationFile::load_file(){
  Message::show("LF", "Loading file...");

  bool cont;
  cont = true;

  if (!opened){
    open_read();
  }

  
  while (cont){
    this->next();
    
    // check if we have to continue loading...    
    // We continue if next_type is not "End" and is not EOF.
    cont = (next_type != LF_ENDFILE) && (! file.eof()) && opened;
  }

  close();
}

/*********************
File manipulation methods
*********************/

void LocationFile::open_read(){ 

  file.open(filename.c_str(), ios::in);    
  opened = true;

  Message::show("LF","File opened:");
  Message::show("LF",filename.c_str());
}

void LocationFile::close(){
  if (opened){
    file.close();
    Message::show("LF","File closed.");
    opened = false;
  }
}
