/*
  MyKeyListener.hpp
  Copyright (C) 2013  Giménez, Christian N.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  lunes 14 de enero del 2013     */


#ifndef __MYKEYLISTENER_HPP
#define __MYKEYLISTENER_HPP 1

#include <OISKeyboard.h>
#include <OISEvents.h>
#include <OISInputManager.h>
#include <OISMouse.h>

#include "TinyOgre.h"
class TinyOgre;

// using namespace std;

/**
   I'm a MyKeyListener class.
   
   I implement the OIS::KeyListener callbacks for responding when a key is pressed.
 */
class MyKeyListener : public OIS::KeyListener {
  
protected:
  /**

     @attention No debería estar este atributo, aunque si se quita se pierde una gran cantidad de funcionalidad. Si el diseño lo permite, sería ideal retirarlo.

     @todo Si el diseño lo permite, retirar este atributo.

     Óbservese que sería una relación redundante tener este puntero a TinyOgre. 

     La clase principal no debería estar aquí si no fuese porque no se puede redefinir los métodos keyPressed() o keyReleased() para incorporar un parámetro más.

     Si se encontrase un diseño en el cual se pueda eliminar este puntero, sería lo ideal.
   */
  TinyOgre* tiny;

  /**
     Último evento ejecutado.
   */
  OIS::KeyEvent* last_key_event;

public:
  /**
     Como soy una clase genérica, que sólo quiere mostrar cómo funcionar no sé que voy a hacer cuando presionen una tecla.

     Por el momento será mover la cámara, y nada más, pero ¿y si debo hacer caminar un personaje?

     Prefiero pedir el Ogre::Root, pero si más adelante requiero algo más específico, lo ideal sería pedir sólo lo necesario.
   */
  MyKeyListener(TinyOgre *tiny);
  ~MyKeyListener();

  virtual bool keyPressed(const OIS::KeyEvent &ke);
  virtual bool keyReleased(const OIS::KeyEvent &ke);
  
  /**
     Esta función debe ser ejecutada por cada frame. Puede estar en el método FrameListener::frameRenderingQueue(). (En este caso en el EventManager::frameRenderingQueue() ).
    
     Cuando una tecla se presiona sólo se captura cuando se presiona y cuando se libera. Para revisar si se deja presionada keyPressed guarda en el atributo last_key_event el último evento y repeatKeyEvent() lo revisa y vuelve a ejecutarlo.
   */
  void repeatKeyEvent();

  /**
     Setea el último evento de teclado encontrado.
   */
  void setLastKeyEvent(OIS::KeyEvent ke);

};

#endif /* __MYKEYLISTENER_HPP */

