
    // World.cpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#include "World.hpp"
#include "LocationFile.hpp"
#include "LocationBuilder.hpp"
//#include "Light.hpp"

using namespace Loaders;

World::World(string name, string starting_loc, Ogre::SceneManager *smgr){
  this->name = name;
  this->current_loc = NULL;
  this->col_locations = new std::vector<string>();
  this->set_initial_location(starting_loc);

  load_location(starting_loc);
}

World::World(string name){
  this->name = name;
  this->starting = "";

  this->current_loc = NULL;

  this->col_locations = new std::vector<string>();
}

World::~World(){
  if (current_loc != NULL){
    delete(current_loc);
  }
}

bool World::load_location(string loc_name){
  Message::show("W", "load_location(string)");
  Message::show("W", "Loading location...");
  Message::show("W", loc_name.c_str());

  list_locations();

  if (!exists_location(loc_name)){
    Warning::show("W", "Starting location is not in list of locations... aborting loading...");
    return false ;
  }

  if (current_loc != NULL){
    // Current location is unnecessary
    delete(current_loc);
    current_loc = NULL;
  } 

  // Load the new location from file
  LocationBuilder *builder = new LocationBuilder();
  LocationFile *loc = new LocationFile(builder, this->name, loc_name);

  loc->open_read();
  loc->load_file();   
  current_loc = builder->get_location();
  loc->close();  

  // Delete Auxiliary objects
  delete(builder);
  delete(loc);

  return true;
}

void World::load(Ogre::SceneManager *smgr){
  if (starting != ""){
    // Load Ogre things from location 
    current_loc->load(smgr);
  }else{
    Warning::show("W", "Starting location not setted!");
  }
}

Location* World::get_current_loc(){
  return current_loc;
}

void World::set_initial_location(string name){
  this->starting = name;

  //If it doesn't exists in the list, add it.
  add_location(name);  
}

// IMeshSceneNode* World::get_current_node(){
//   current_loc->get_node();
// }

// void World::set_initial_position(ISceneNode* node){
//   current_loc->set_initial_position(node);
// }

void World::add_location(string name){
  Message::show("W", "add_location(string)");

  if (!exists_location(name)){
    // It doesn't exists... add it to the list.
    col_locations->push_back(name);
  }
}


bool World::exists_location(string name){
  Message::show("W", "exists_location(string)");

  if (col_locations->empty()){
    Message::show("W", "list empty... Doesn't exists");
    // If empty, then it will not be founded.
    return false;
  }

  std::vector<string>::iterator it = col_locations->begin();

  string aux = "";
  Message::show("W", aux);

  bool found = false;    

  // While it is not found and it is not at the end...
  while ((not found) and (it != col_locations->end())){
    aux = *it;
    Message::show("W", aux);
    
    found = aux == name; // is it found?
    
    ++it;    
  }
  
  return found;       
}

string World::get_name(){
  return name;
}

void World::list_locations(){
  Message::show("W", "list_locations()");
  
  std::vector<string>::iterator i;

  std::cout << "Listing..." << endl;
  for (i = col_locations->begin(); i != col_locations->end(); ++i){
    std::cout << *i << endl;
  }
}
