
    // World.hpp
    // Copyright (C) 2013  Giménez, Christian N.

    // This program is free software: you can redistribute it and/or modify
    // it under the terms of the GNU General Public License as published by
    // the Free Software Foundation, either version 3 of the License, or
    // (at your option) any later version.

    // This program is distributed in the hope that it will be useful,
    // but WITHOUT ANY WARRANTY; without even the implied warranty of
    // MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    // GNU General Public License for more details.

    // You should have received a copy of the GNU General Public License
    // along with this program.  If not, see <http://www.gnu.org/licenses/>.

    // Miércoles 23 De Enero Del 2013    


#ifndef __WORLD_HPP__
#define __WORLD_HPP__

#include <vector>

using namespace std;

#include "Location.hpp"
#include "LocationBuilder.hpp"
//#include "LocationFile.hpp"

using namespace Loaders;

/**
   I represent a complete world!

   For now I support only a starting location, but in a near future I'll add more!
 */
class World{

protected:
  /**
     This store my name!
   */
  string name;
  /**
     This is the starting location name.
   */
  string starting;
  
  /**
     This is the current location.
   */
  Location* current_loc;

  /**
     This is a list of locations we can load.
     Is a Hash that associates a stringc instance with its Location*.

     The main idea is that we don't want to search for the location name, if we put a location name we get it directly.
  */
  
  std::vector<string>* col_locations;
  //std::map<string, Location*> col_locations;
  

public:
  /**
     Create a new instance of World. 

     Load the starting location into Ogre.
     
     @param name An instance of string with the name of the world.
     @param starting_loc An instance of stringc with the name of the starting Location. If not setted we use the one defined in STARTING_LOC_NAME. 
     @param smgr A instance of Ogre::SceneManager in which to load the starting location.
  */
  World(string name, string starting_loc, Ogre::SceneManager *smgr);
  /**
     Load an empty world... 

     Use this constructor if you want no location to be loaded into Ogre. You must use change_location() if you want to load a new location.

     @param name An instance of string with the name of the world.
   */
  World(string name);
  ~World();

  /**
     Change the current location loading what is necessary and preparing it for loading into the Ogre::SceneManager.

     @param name is an instance of stringc with the name of the new location.
     
     @retval true if it was loaded.
     @retval false if the location doesn't exists in this world.
   */
  bool load_location(string name);  

  /**
     I load the actual location of this world. If the current location is not defined, I load the starting location!

     @param smgr An instance of SceneManager for loading the current location into it.
   */
  void load(Ogre::SceneManager *smgr);

  /**
     Add a location name into my list of locations availables.

     The user can use any of this locations to load because these are in my world.

     @param name an instance of String with the location name.
   */
  void add_location(string name);

  /**
     I search for a location name, if it exists, I return true.

     @param name The name of the location to search.

     @retval true If the location is in this world.
     @retval false If the location is not in this world.
   */
  bool exists_location(string name);

  /**
     Add this location as the initial location of this world.
     
     @param name An instance of string with the location name.
   */
  void set_initial_location(string name);

  /**
     Get the current location node.
     
     @return An instance of IMeshSceneNode that is the node of the current location.
   */
  //IMeshSceneNode* get_current_node();

  /**
     Set the initial position of the actual location to the node.

     @param node An instance of ISceneNode to change its position.
   */
  //void set_initial_position(ISceneNode* node);

  /**
     @name Getters and Setters
   */
  //@{
  /**
     Return the current location, the one that I loaded before.

     @return A Location pointer to an instance that is the current location.
   */
  Location* get_current_loc();
  
  string get_name();
  
  //@}


  /**
     @name For Debugging purposes.
  */
  //@{
  void list_locations();
  //@}
};

#endif
