/*
  WorldBuilder.hpp
  Copyright (C) 2013  Giménez, Christian N.
  
  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>.
  
  sábado 26 de enero del 2013     */


#ifndef __WORLDBUILDER_HPP
#define __WORLDBUILDER_HPP 1

#include <string>

using namespace std;

#include "World.hpp"

namespace Loaders{

  /**
     I'm responsible for creating a new World with all the information.
  */
  class WorldBuilder {

  protected:
    
    /**
       This is the world instance that I'm building.
    */
    World* world;

  public:    
    WorldBuilder();
    ~WorldBuilder();

    /**
       Return the recently created World, if you requested one.

       Please, use build_world first.
     */
    World* get_world();

    
    /**
       I create a new instance of a World with the basic information. You can retrieve it with get_world() or fill it with more information using the others messages implemented in my class.

       @param name An instance of string with the name of the world.
       @param initial_location An instance of string with the initial location name(All world have to have at least one location).
     */
    void build_world(string name, string initial_location);

    /**
       Add a Location name to the current created world.

       @param name An instance of string with the Location name.
     */
    void add_location(string name);
    
  };

}
#endif /* __WORLDBUILDER_HPP */

